@@ -8,7 +8,9 @@ void (*f_anthy_get_segment)(anthy_context_t ac, int,int,char *, int);
 void (*f_anthy_get_segment_stat)(anthy_context_t ac, int, struct anthy_segment_stat *);
 void (*f_anthy_commit_segment)(anthy_context_t ac, int, int);
 void (*f_anthy_set_string)(anthy_context_t ac, char *);
-
+extern int eng_ph;
+extern gint64 key_press_time;
+static GtkWidget *event_box_anthy;
 
 struct {
   char *en;
@@ -219,7 +221,7 @@ struct {
 {"n'",	"ん"},
 {"nn",	"ん"},
 {"n",	"ん"},
-{"m",	"ん"},
+{"m",	"ん"},  // tombo
 {"wyi",	"ゐ"},
 {"wye",	"ゑ"},
 {",",	"、"},
@@ -248,10 +250,21 @@ struct {
 {"}",	"｝"},
 {"|",	"｜"},
 {"'",	"’"},
+{"\"",	"”"},
 {"`",	"‘"},
 {"?",	"？"},
 {":",	"："},
 {";",	"；"},
+{"0",	"0"},
+{"1",	"1"},
+{"2",	"2"},
+{"3",	"3"},
+{"4",	"4"},
+{"5",	"5"},
+{"6",	"6"},
+{"7",	"7"},
+{"8",	"8"},
+{"9",	"9"},
 };
 
 
@@ -546,12 +559,40 @@ static void next_page()
   disp_select();
 }
 
+
+static int flush_input()
+{
+  hide_selections_win();
+
+  int val;
+  if (state==STATE_CONVERT) {
+    val = TRUE;
+    send_seg();
+  } else {
+    val = send_jp();
+  }
+
+  jpN=0;
+  keysN = 0;
+  cursor = 0;
+  disp_input();
+  state = STATE_ROMANJI;
+  auto_hide();
+  return val;
+}
+
 gboolean feedkey_anthy(int kv, int kvstate)
 {
   int lkv = tolower(kv);
   int shift_m=(kvstate&ShiftMask) > 0;
 //  printf("%x %c  %d\n", kv, kv, shift_m);
 
+  if (kv==XK_Shift_L||kv==XK_Shift_R)
+    key_press_time = current_time();
+
+  if (!eng_ph)
+    return 0;
+
   gboolean is_empty = !keysN && !jpN && !segN;
 
 //  printf("empty %d\n", is_empty);
@@ -561,23 +602,7 @@ gboolean feedkey_anthy(int kv, int kvstate)
       if (is_empty)
         return FALSE;
 send:
-      hide_selections_win();
-
-      int val;
-      if (state==STATE_CONVERT) {
-        val = TRUE;
-        send_seg();
-      } else {
-        val = send_jp();
-      }
-
-      jpN=0;
-      keysN = 0;
-      cursor = 0;
-      disp_input();
-      state = STATE_ROMANJI;
-      auto_hide();
-      return val;
+      return flush_input();
     case XK_Escape:
         if (state==STATE_SELECT) {
           state = STATE_CONVERT;
@@ -773,6 +798,23 @@ lab1:
   return TRUE;
 }
 
+static void mouse_button_callback( GtkWidget *widget,GdkEventButton *event, gpointer data)
+{
+//  dbg("mouse_button_callback %d\n", event->button);
+  switch (event->button) {
+    case 1:
+      toggle_win_sym();
+      break;
+    case 2:
+      inmd_switch_popup_handler(widget, (GdkEvent *)event);
+      break;
+    case 3:
+      exec_gcin_setup();
+      break;
+  }
+}
+
+
 #include <dlfcn.h>
 
 int init_win_anthy()
@@ -781,6 +823,8 @@ int init_win_anthy()
   void *handle;
   char *error;
 
+  eng_ph = 1;
+
   if (win_anthy)
     return TRUE;
 
@@ -809,7 +853,12 @@ int init_win_anthy()
   }
 
   if ((*f_anthy_init)() == -1) {
-    printf("cannot init anthy");
+    GtkWidget *dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+                                     GTK_MESSAGE_ERROR,
+                                     GTK_BUTTONS_CLOSE,
+                                     "Cannot init anthy. Please install anthy.");
+    gtk_dialog_run (GTK_DIALOG (dialog));
+    gtk_widget_destroy (dialog);
     return FALSE;
   }
 
@@ -836,17 +885,19 @@ int init_win_anthy()
   win_anthy = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_default_size(GTK_WINDOW (win_anthy), 40, 50);
 
+
   gtk_widget_realize (win_anthy);
   set_no_focus(win_anthy);
 
-  g_signal_connect (G_OBJECT (win_anthy), "delete_event",
-                    G_CALLBACK (exit), NULL);
+  event_box_anthy = gtk_event_box_new();
+
+  gtk_container_add(GTK_CONTAINER(win_anthy), event_box_anthy);
 
   GtkWidget *hbox_top = gtk_hbox_new (FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(event_box_anthy), hbox_top);
 
-  gtk_container_add(GTK_CONTAINER(win_anthy), hbox_top);
-
-  gtk_widget_show_all(win_anthy);
+  g_signal_connect(G_OBJECT(event_box_anthy),"button-press-event",
+                   G_CALLBACK(mouse_button_callback), NULL);
 
   for(i=0; i < MAX_SEG_N; i++) {
     seg[i].label = gtk_label_new(NULL);
@@ -854,9 +905,10 @@ int init_win_anthy()
     gtk_box_pack_start (GTK_BOX (hbox_top), seg[i].label, FALSE, FALSE, 0);
   }
 
+  gtk_widget_show_all(win_anthy);
+
   create_win1();
   create_win1_gui();
-
   change_anthy_font_size();
 
   if (!phkbm.selkeyN)
@@ -867,14 +919,20 @@ int init_win_anthy()
   return TRUE;
 }
 
-void show_win_anthy()
+int anthy_visible()
 {
-  if (!gcin_pop_up_win || !is_empty()) {
-    if (!GTK_WIDGET_VISIBLE(win_anthy))
-      gtk_widget_show(win_anthy);
-  }
+  return GTK_WIDGET_VISIBLE(win_anthy);
 }
 
+extern gboolean force_show;
+void show_win_anthy()
+{
+  if (!gcin_pop_up_win || !is_empty() || force_show) {
+    if (!anthy_visible())
+      gtk_widget_show(win_anthy);
+    show_win_sym();
+  }
+}
 
 void hide_win_anthy()
 {
@@ -883,6 +941,7 @@ void hide_win_anthy()
     hide_selections_win();
   }
   gtk_widget_hide(win_anthy);
+  hide_win_sym();
 }
 
 void change_anthy_font_size()
@@ -890,17 +949,18 @@ void change_anthy_font_size()
   GdkColor fg;
   gdk_color_parse(gcin_win_color_fg, &fg);
   change_win_bg(win_anthy);
+  change_win_bg(event_box_anthy);
 
   int i;
   for(i=0; i < MAX_SEG_N; i++) {
     GtkWidget *label = seg[i].label;
     set_label_font_size(label, gcin_font_size);
-    if (gcin_win_color_use)
+    if (gcin_win_color_use) {
       gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &fg);
+    }
   }
 }
 
-
 void move_win_anthy(int x, int y)
 {
 #if 0
@@ -925,3 +985,28 @@ void move_win_anthy(int x, int y)
 
   move_win_sym();
 }
+
+
+int feedkey_anthy_release(KeySym xkey, int kbstate)
+{
+  switch (xkey) {
+     case XK_Shift_L:
+     case XK_Shift_R:
+        if (
+(  (tsin_chinese_english_toggle_key == TSIN_CHINESE_ENGLISH_TOGGLE_KEY_Shift) ||
+   (tsin_chinese_english_toggle_key == TSIN_CHINESE_ENGLISH_TOGGLE_KEY_ShiftL
+     && xkey == XK_Shift_L) ||
+   (tsin_chinese_english_toggle_key == TSIN_CHINESE_ENGLISH_TOGGLE_KEY_ShiftR
+     && xkey == XK_Shift_R))
+          &&  current_time() - key_press_time < 300000) {
+          flush_input();
+          key_press_time = 0;
+          hide_selections_win();
+          tsin_set_eng_ch(!eng_ph);
+          return 1;
+        } else
+          return 0;
+     default:
+        return 0;
+  }
+}
