@@ -1,6 +1,8 @@
 #include <signal.h>
 // #include <openssl/ssl.h>
 #include "gcin.h"
+#include "config.h"
+#include <libintl.h>
 
 Display *dpy;
 Window root;
@@ -8,14 +10,13 @@ int win_xl, win_yl;
 int win_x, win_y;   // actual win x/y
 int dpy_xl, dpy_yl;
 
-#if USE_XIM
-gboolean dual_xim=TRUE;
-DUAL_XIM_ENTRY xim_arr[2];
-DUAL_XIM_ENTRY *pxim_arr;
-#else
 DUAL_XIM_ENTRY xim_arr[1];
+
+#if USE_XIM
+DUAL_XIM_ENTRY *pxim_arr;
 #endif
 
+gboolean win_kbm_inited;
 
 char *fullchar[]=
 {"　","！","”","＃","＄","％","＆","’","（","）","＊","＋",
@@ -43,31 +44,16 @@ void start_inmd_window()
   switch (default_input_method) {
     case 3:
       create_win_pho();
-#if USE_XIM
-      if (dual_xim)
-        create_win0();
-      xim_arr[1].xim_xwin = xwin0;
-#endif
       xim_arr[0].xim_xwin = xwin_pho;
       break;
+#if USE_TSIN
     case 6:
       create_win0();
-#if USE_XIM
-      if (dual_xim)
-        create_win1();
-      xim_arr[1].xim_xwin = xwin1;
-#endif
       xim_arr[0].xim_xwin = xwin0;
       break;
+#endif
     default:
       create_win_gtab();
-
-#if USE_XIM
-      if (dual_xim)
-        create_win_pho();
-      xim_arr[1].xim_xwin = xwin_pho;
-#endif
-
       xim_arr[0].xim_xwin = xwin_gtab;
       break;
   }
@@ -106,6 +92,7 @@ static XIMTriggerKey trigger_keys[] = {
 /* Supported Encodings */
 static XIMEncoding chEncodings[] = {
         "COMPOUND_TEXT",
+        0
 };
 static XIMEncodings encodings;
 
@@ -146,20 +133,11 @@ int xim_gcin_FocusOut(IMChangeFocusStruct *call_data);
 
 #define DEBUG 0
 
-
 int gcin_ProtoHandler(XIMS ims, IMProtocol *call_data)
 {
 //  dbg("gcin_ProtoHandler %x ims\n", ims);
   int index=0;
 
-  if (ims == xim_arr[0].xims)
-    index = 0;
-  else
-  if (ims == xim_arr[1].xims)
-    index = 1;
-  else
-     p_err("bad ims %x\n", ims);
-
   pxim_arr = &xim_arr[index];
   switch_IC_index(index);
 
@@ -172,8 +150,10 @@ int gcin_ProtoHandler(XIMS ims, IMProtocol *call_data)
 #define MAX_CONNECT 20000
     {
       IMOpenStruct *pimopen=(IMOpenStruct *)call_data;
+
       if(pimopen->connect_id > MAX_CONNECT - 1)
         return True;
+
 #if DEBUG
     dbg("open lang %s  connectid:%d\n", pimopen->lang.name, pimopen->connect_id);
 #endif
@@ -271,29 +251,24 @@ void open_xim()
   triggerKeys.count_keys = sizeof(trigger_keys)/sizeof(trigger_keys[0]);
   triggerKeys.keylist = trigger_keys;
 
-  encodings.count_encodings = sizeof(chEncodings)/sizeof(XIMEncoding);
+  encodings.count_encodings = sizeof(chEncodings)/sizeof(XIMEncoding) - 1;
   encodings.supported_encodings = chEncodings;
 
-
-  int dualN = dual_xim ? 2 : 1;
-  int i;
-
-  for(i=0; i < dualN; i++) {
-    if ((xim_arr[i].xims = IMOpenIM(dpy,
-            IMServerWindow,         xim_arr[i].xim_xwin,        //input window
-            IMModifiers,            "Xi18n",        //X11R6 protocol
-            IMServerName,           xim_arr[i].xim_server_name, //XIM server name
-            IMLocale,               xim_arr[i].server_locale,  //XIM server locale
-            IMServerTransport,      "X/",      //Comm. protocol
-            IMInputStyles,          &im_styles,   //faked styles
-            IMEncodingList,         &encodings,
-            IMProtocolHandler,      gcin_ProtoHandler,
-            IMFilterEventMask,      KeyPressMask,
-            IMOnKeysList, &triggerKeys,
-  //        IMOffKeysList, &triggerKeys,
-            NULL)) == NULL) {
-            p_err("IMOpenIM failed. Maybe another XIM server is running.\n");
-    }
+  if ((xim_arr[0].xims = IMOpenIM(dpy,
+          IMServerWindow,         xim_arr[0].xim_xwin,        //input window
+          IMModifiers,            "Xi18n",        //X11R6 protocol
+          IMServerName,           xim_arr[0].xim_server_name, //XIM server name
+          IMLocale,               xim_arr[0].server_locale,  //XIM server locale
+          IMServerTransport,      "X/",      //Comm. protocol
+          IMInputStyles,          &im_styles,   //faked styles
+          IMEncodingList,         &encodings,
+          IMProtocolHandler,      gcin_ProtoHandler,
+          IMFilterEventMask,      KeyPressMask,
+          IMOnKeysList, &triggerKeys,
+//        IMOffKeysList, &triggerKeys,
+          NULL)) == NULL) {
+          p_err("IMOpenIM '%s' failed. Maybe another XIM server is running.\n",
+          xim_arr[0].xim_server_name);
   }
 }
 
@@ -309,12 +284,15 @@ static void reload_data()
 {
   dbg("reload_data\n");
   load_setttings();
-  load_tsin_db();
+//  load_tsin_db();
   change_win0_style();
   change_win_gtab_style();
-  change_win_pho_style();
+//  change_win_pho_style();
   load_tab_pho_file();
   change_tsin_color();
+  if (win_kbm_inited)
+    update_win_kbm();
+  gcb_main();
 }
 
 void change_tsin_font_size();
@@ -332,7 +310,7 @@ static void change_font_size()
   change_win_sym_font_size();
   change_win0_style();
   change_win_gtab_style();
-  change_win_pho_style();
+//  change_win_pho_style();
 }
 
 static int xerror_handler(Display *d, XErrorEvent *eve)
@@ -340,21 +318,42 @@ static int xerror_handler(Display *d, XErrorEvent *eve)
   return 0;
 }
 
+
 Atom gcin_atom;
+void cb_trad_sim_toggle()
+{
+  toggle_gb_output();
+#if TRAY_ENABLED
+  update_tray_icon();
+#endif
+}
+void execute_message(char *message);
+int b_show_win_kbm=0;
+void kbm_toggle()
+{
+  win_kbm_inited = 1;
+  b_show_win_kbm^=1;
+  if (b_show_win_kbm)
+    show_win_kbm();
+  else
+    hide_win_kbm();
+}
 
 static GdkFilterReturn my_gdk_filter(GdkXEvent *xevent,
                                      GdkEvent *event,
                                      gpointer data)
 {
    XEvent *xeve = (XEvent *)xevent;
-
+#if 0
+   dbg("a zzz %d\n", xeve->type);
+#endif
    if (xeve->type == PropertyNotify) {
      XPropertyEvent *xprop = &xeve->xproperty;
 
      if (xprop->atom == gcin_atom) {
        Atom actual_type;
        int actual_format;
-       u_long nitems,bytes_after;
+       unsigned long nitems,bytes_after;
        char *message;
 
        if (XGetWindowProperty(dpy, xprop->window, gcin_atom, 0, 64,
@@ -370,19 +369,35 @@ static GdkFilterReturn my_gdk_filter(GdkXEvent *xevent,
          change_font_size();
        } else
        if (!strcmp(message, GB_OUTPUT_TOGGLE)) {
-         extern gboolean gb_output;
-         gb_output = !gb_output;
+         cb_trad_sim_toggle();
        } else
+       if (!strcmp(message, KBM_TOGGLE)) {
+         kbm_toggle();
+       } else
+       if (strstr(message, "#gcin_message")) {
+         execute_message(message);
+       }
+       else
          reload_data();
 
        XFree(message);
        return GDK_FILTER_REMOVE;
      }
    }
+
+   // only very old WM will enter this
+   if (xeve->type == FocusIn || xeve->type == FocusOut) {
+#if 0
+     dbg("focus %s\n", xeve->type == FocusIn ? "in":"out");
+#endif
+     return GDK_FILTER_REMOVE;
+   }
+
 #if USE_XIM
    if (XFilterEvent(xeve, None) == True)
      return GDK_FILTER_REMOVE;
 #endif
+
    return GDK_FILTER_CONTINUE;
 }
 
@@ -396,8 +411,8 @@ void init_atom_property()
 
 void hide_win0();
 void do_exit();
-void destory_win0();
-void destory_win1();
+void destroy_win0();
+void destroy_win1();
 void destroy_win_gtab();
 void free_pho_mem(),free_tsin(),free_all_IC(), free_gtab(), free_phrase();
 
@@ -414,97 +429,71 @@ void do_exit()
   free_phrase();
 
 #if 1
-  destory_win0();
-  destory_win1();
+  destroy_win0();
+  destroy_win1();
   destroy_win_gtab();
 #endif
 
   gtk_main_quit();
 }
 
-
-static void exec_script(char *name)
-{
-  char scr[128];
-
-  sprintf(scr, GCIN_SCRIPT_DIR"/%s", name);
-  system(scr);
-}
-
-static void exec_setup_scripts()
-{
-  exec_script("gcin-user-setup "GCIN_TABLE_DIR" "GCIN_BIN_DIR);
-}
-
-
 char *get_gcin_xim_name();
 void load_phrase(), init_TableDir(),  load_gtab_list();
-void init_im_serv();
+void init_im_serv(), init_tray();
 void init_gcin_im_serv(Window win);
 
 int main(int argc, char **argv)
 {
+
+#if GCIN_i18n_message
+  gtk_set_locale();
+  bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+  textdomain(GETTEXT_PACKAGE);
+#endif
+
 #if USE_XIM
-  dual_xim = getenv("GCIN_DUAL_XIM_OFF") == NULL;
   char *lc_ctype = getenv("LC_CTYPE");
   char *lc_all = getenv("LC_ALL");
   char *lang = getenv("LANG");
-  if (!lc_ctype)
-    lc_ctype = "";
-  if (!lc_all)
-    lc_all = "";
+
   dbg("gcin get env LC_CTYPE=%s  LC_ALL=%s  LANG=%s\n", lc_ctype, lc_all, lang);
 
-  char *enc;
-  char *lc_enc;
-  char *lc_utf8;
+  if (!lc_ctype && lang)
+    lc_ctype = lang;
+
+  if (lc_all)
+    lc_ctype = lc_all;
+
+  if (!lc_ctype)
+    lc_ctype = "zh_TW.Big5";
+
   char *lc;
 
-  if (lc_ctype && strstr(lc_ctype, "en") /* || lang && strstr(lang, "en") */){
-    lc = "en_US";
-    enc = "UTF-8";
-#if 0
-    lc_enc = "zh_TW.UTF-8";
-    lc_utf8 = "en_US.UTF-8";
+  signal(SIGCHLD, SIG_IGN);
+  signal(SIGPIPE, SIG_IGN);
+
+  char *t = strchr(lc_ctype, '.');
+  if (t) {
+    int len = t - lc_ctype;
+#if MAC_OS || FREEBSD
+    lc = strdup(lc_ctype);
+    lc[len] = 0;
 #else
-    // XIM in en_US doesn't work
-    lc_enc = "zh_TW.UTF-8";
-    lc_utf8 = "zh_TW.UTF-8";
+    lc = strndup(lc_ctype, len);
 #endif
-  } else
-  if (lc_ctype && strstr(lc_ctype, "CN") || lang && strstr(lang, "CN")) {
-    lc = "zh_CN";
-    enc = "GB2312";
-    lc_enc = "zh_CN.GB2312";
-    lc_utf8 = "zh_CN.UTF-8";
-  } else {
-    lc = "zh_TW";
-    enc = "Big5";
-    lc_enc = "zh_TW.Big5";
-    lc_utf8 = "zh_TW.UTF-8";
   }
+  else
+    lc = lc_ctype;
 
   xim_arr[0].server_locale = lc;
   char *xim_server_name = get_gcin_xim_name();
 
   strcpy(xim_arr[0].xim_server_name, xim_server_name);
-  strcpy(xim_arr[1].xim_server_name, xim_server_name);
 
-  if ((lc_ctype && strstr(lc_ctype, ".UTF-8")) ||
-      (lc_all && strstr(lc_all, ".UTF-8")) ||
-      (lang && strstr(lang, "UTF-8"))) {
-    xim_arr[0].b_send_utf8_str = TRUE;
-    xim_arr[1].b_send_utf8_str = FALSE;
-    xim_arr[1].server_locale = lc_enc;
-    strcat(xim_arr[1].xim_server_name, enc);
-    dbg("gcin will use UTF-8 as the default encoding\n");
-  } else {
-    xim_arr[0].b_send_utf8_str = FALSE;
-    xim_arr[1].b_send_utf8_str = TRUE;
-    xim_arr[1].server_locale = lc_utf8;
-    strcat(xim_arr[1].xim_server_name, ".UTF-8");
-    dbg("gcin will use %s as the default encoding\n", enc);
-  }
+#endif
+
+#if USE_XIM
+  dbg("gcin XIM will use %s as the default encoding\n", lc_ctype);
 #endif
 
   if (argc == 2 && (!strcmp(argv[1], "-v") || !strcmp(argv[1], "--version") || !strcmp(argv[1], "-h")) ) {
@@ -524,7 +513,7 @@ int main(int argc, char **argv)
 
   root=DefaultRootWindow(dpy);
   start_inmd_window();
-#if USE_XIM
+#if USE_XIM && 1
   open_xim();
 #endif
   init_atom_property();
@@ -542,6 +531,17 @@ int main(int argc, char **argv)
   init_gcin_im_serv(xim_arr[0].xim_xwin);
 #endif
 
+#if TRAY_ENABLED
+  if (gcin_status_tray) {
+    init_tray();
+  }
+#endif
+
+#if 1
+  if (gcb_position)
+    gcb_main();
+#endif
+
   gtk_main();
 
   return 0;
