@@ -46,8 +46,10 @@ static void shutdown_client(int fd)
 //  dbg("client shutdown rn %d\n", rn);
   gdk_input_remove(gcin_clients[fd].tag);
 
-  if (gcin_clients[fd].cs == current_CS)
+  if (gcin_clients[fd].cs == current_CS) {
+    hide_in_win(current_CS);
     current_CS = NULL;
+  }
 
   free(gcin_clients[fd].cs);
   gcin_clients[fd].cs = NULL;
@@ -89,13 +91,20 @@ void process_client_req(int fd)
   } else {
     cs = gcin_clients[fd].cs;
 
+    int new_cli = 0;
     if (!cs) {
       cs = gcin_clients[fd].cs = tzmalloc(ClientState, 1);
+      new_cli = 1;
     }
 
     cs->client_win = req.client_win;
     cs->b_gcin_protocol = TRUE;
     cs->input_style = InputStyleOverSpot;
+
+    if (gcin_init_im_enabled && new_cli) {
+      current_CS = cs;
+      init_state_chinese(cs);
+    }
   }
 
   if (!cs)
@@ -112,7 +121,7 @@ void process_client_req(int fd)
     case GCIN_req_key_press:
     case GCIN_req_key_release:
       current_CS = cs;
-#if 0
+#if DBG && 0
       {
         char tt[128];
 
@@ -122,7 +131,7 @@ void process_client_req(int fd)
           strcpy(tt, XKeysymToString(req.keyeve.key));
         }
 
-//        dbg_time("GCIN_key_press  %x %s\n", cs, tt);
+        dbg_time("GCIN_key_press  %x %s\n", cs, tt);
       }
 #endif
       to_gcin_endian_4(&req.keyeve.key);
@@ -136,9 +145,13 @@ void process_client_req(int fd)
         status = ProcessKeyRelease(req.keyeve.key, req.keyeve.state);
       }
 
-      if (status) {
+      if (status)
         reply.flag |= GCIN_reply_key_processed;
-      }
+#if 0
+      if (cs->im_state == GCIN_STATE_DISABLED)
+        reply.flag |= GCIN_reply_key_state_disabled;
+#endif
+//      dbg("srv flag %x\n", reply.flag);
 
       int datalen = reply.datalen =
         output_bufferN ? output_bufferN + 1 : 0; // include '\0'
@@ -175,8 +188,18 @@ void process_client_req(int fd)
       break;
     case GCIN_req_set_flags:
       if (BITON(req.flag, FLAG_GCIN_client_handle_raise_window)) {
-        cs->b_raise_window = TRUE;
+#if DBG
+        dbg("********* raise * window\n");
+#endif
+        if (!gcin_pop_up_win)
+          cs->b_raise_window = TRUE;
       }
+
+      int rflags = 0;
+      if (gcin_pop_up_win)
+        rflags = FLAG_GCIN_srv_ret_status_use_pop_up;
+
+      write_enc(fd, &rflags, sizeof(rflags));
       break;
     default:
       dbg_time("Invalid request %x from:", req.req_no);
@@ -187,8 +210,12 @@ void process_client_req(int fd)
 
       if (!getpeername(fd, (struct sockaddr *)&addr, &len)) {
         u_char addr_ch[4];
+#if 0
         memcpy(addr_ch, &addr.sin_addr.s_addr,sizeof(addr.sin_addr.s_addr));
         dbg("%d %d %d %d\n", addr_ch[0], addr_ch[1], addr_ch[2], addr_ch[3]);
+#else
+        dbg("%s\n", inet_ntoa(addr));
+#endif
       } else {
         perror("getpeername\n");
       }
@@ -197,3 +224,13 @@ void process_client_req(int fd)
       break;
   }
 }
+
+
+
+void close_all_clients()
+{
+  int i;
+  for(i=3; i <= gcin_clientsN; i++)
+    if (gcin_clients[i].tag)
+      close(i);
+}
